#ifndef GUIDANCE_H_
#define GUIDANCE_H_

#include "Pose_SE2.hpp"

using namespace teb_local_planner;

class Guidance
{
public:

    double sampleTime;
    int idx; // next waypoint index
    double goal_dist;
    double switch_dist;
    double y_e_int;
    double Kp;
    double Ki;
    int reached_goal;

    struct Reference
    {
      double r_max;
      double alpha_max;
      double w_psi_d;
      double zeta_psi_d;

      double u_max;
      double a_max;
      double w_u_d;
      double zeta_u_d;
    } ref;

    struct COLAVReference
    {
      double r_max;
      double alpha_max;
      double w_psi_d;
      double zeta_psi_d;

      double u_max;
      double a_max;
      double w_u_d;
      double zeta_u_d;
    } cref;

    std::vector<Eigen::Vector2d> waypoints_;

    Guidance(std::vector<Eigen::Vector2d>& waypoints);

    void headingRefModel(double& psi_ref, double& psi_d, double& r_d, double& alpha_d);

    void speedRefModel(double& u_ref, double& u_d, double& a_d);

    double LOS(PoseSE2& eta);

    void referenceModel(double& psi_ref, double& psi_d, double& r_d, double& alpha_d, double u_ref, double& u_d, double& a_d)
    {
      headingRefModel(psi_ref, psi_d, r_d, alpha_d);
      speedRefModel(u_ref, u_d, a_d);
    }

    void COLAVheadingRefModel(double& psi_ref, double& psi_d, double& r_d, double& alpha_d);

    void COLAVspeedRefModel(double& u_ref, double& u_d, double& a_d);

    void COLAVreferenceModel(double& psi_ref, double& psi_d, double& r_d, double& alpha_d, double u_ref, double& u_d, double& a_d)
    {
      COLAVheadingRefModel(psi_ref, psi_d, r_d, alpha_d);
      COLAVspeedRefModel(u_ref, u_d, a_d);
    }

    std::pair<PoseSE2, Eigen::Vector3d> getLocalGoal(PoseSE2& eta, double& u_ref);

public:
  EIGEN_MAKE_ALIGNED_OPERATOR_NEW
};

#endif