#ifndef TANKER_H_
#define TANKER_H_

#include "Pose_SE2.hpp"
#include <boost/optional.hpp>

using namespace teb_local_planner;

class Tanker
{
  public:
    // ship parameters
    double L;
    double T;
    double h;
    double delta_max;
    double Ddelta_max;
    double n_max;
    double Dn_max;
    double t;
    double cun;
    double cnn;
    double Tuu;
    double Tun;
    double Tnn;
    double d11;
    double d22;
    double d33;
    double YT;
    double NT;
    double Xuu;
    double Yvv;
    double Nvr;
    double Xvv;
    double Yuv;
    double Nuv;
    double Xudotz;
    double Yvdotz;
    double Nrdotz;
    double Xuuz;
    double Yurz; 
    double Nurz;
    double Xvrz;
    double Yvvz; 
    double Nvrz;
    double Xccdd;
    double Yuvz;
    double Nuvz; 
    double Xccbd;
    double Yccd;
    double Nccd;
    double Xvvzz; 
    double Yccbbd;
    double Nccbbd;
    double Yccbbdz;
    double Nccbbdz;
    double z;
    double m11;
    double m22;
    double m33;
    // sample time
    double sampleTime;

  struct Controller
  {
    double m_psi;
    double d_psi;
    double k_psi;
    double wn_psi;
    double zeta_psi;
    double e_psi_i;

    double K_sigma;
    double e_u_i;
    double phi;
  } control;

  Tanker();

  void headingControl(double& delta_c, double psi, double psi_d, double r, boost::optional<double> r_d);

  void speedControl(double& n_c, double u, double u_d, boost::optional<double> u_d_dot);

  void dynamics(Eigen::Vector3d& nu, double& delta, double& n, double delta_c, double n_c);

  void kinematics(PoseSE2& eta, Eigen::Vector3d nu);

  void simulate(PoseSE2& eta, Eigen::Vector3d& nu, double& delta, double& n, double& delta_c, double& n_c, double psi_d, double u_d, boost::optional<double> r_d, boost::optional<double> u_d_dot);


public:
  EIGEN_MAKE_ALIGNED_OPERATOR_NEW
};

#endif