#ifndef USV_H_
#define USV_H_

#include "Pose_SE2.hpp"
#include "misc.hpp"
#include <boost/optional.hpp>

using namespace teb_local_planner;

class USV
{
    public:
        double g;
        double rho;
        double T_n;
        double L;
        double B;
        double m;
        double mp;
        double m_total;
        Eigen::Vector3d rp;
        Eigen::Vector3d rg;
        Eigen::Matrix3d S_rg;
        Eigen::MatrixXd H_rg;
        Eigen::Matrix3d S_rp;
        double R44;
        double R55;
        double R66;
        double T_yaw;
        double U_max;

        double B_pont;
        double y_pont;
        double Cw_pont;
        double Cb_pont;
        double nabla;
        double T;
        Eigen::Matrix3d Ig_CG;
        Eigen::Matrix3d Ig;

        double l1;
        double l2;
        double k_pos;
        double k_neg;
        double n_max;
        double n_min;
        Eigen::MatrixXd MRB_CG;
        Eigen::MatrixXd MRB;

        double Xudot;
        double Yvdot;
        double Zwdot;
        double Kpdot;
        double Mqdot;
        double Nrdot;

        Eigen::MatrixXd MA;
        Eigen::MatrixXd M;
        Eigen::MatrixXd Minv;

        double Aw_pont;
        double I_T;
        double I_L;
        double KB;
        double BM_T;
        double BM_L;
        double KM_T;
        double KM_L;
        double KG;
        double GM_T;
        double GM_L;

        double G33;
        double G44;
        double G55;
        Eigen::MatrixXd G_CF;
        double LCF;
        Eigen::MatrixXd H;
        Eigen::MatrixXd G;

        double w3;
        double w4;
        double w5;

        double Xu;
        double Yv;
        double Zw;
        double Kp;
        double Mq;
        double Nr;

        Eigen::MatrixXd D;

        double trim_moment;
        double trim_setpoint;

        Eigen::Matrix2d B_config;
        Eigen::Matrix2d Binv;

        double e_psi;
        double e_int_psi;
        double wn_psi;
        double zeta_psi;

        double e_u;
        double e_int_u;

        double V_c;
        double beta_c;

        double K_sigma;
        double phi;

        double sampleTime;

        USV(double V_current, double beta_current);

        double headingControl(double& psi, double& psi_d, double& r, double& r_d);

        double speedControl(double& u, double& u_d, double& u_d_dot);

        void controlAllocation(double& tau_X, double& tau_N, Eigen::Vector2d& n_c);

        void dynamics(Eigen::VectorXd& eta, Eigen::VectorXd& nu, Eigen::Vector2d& n, Eigen::Vector2d& n_c);

        void kinematics(Eigen::VectorXd& eta, Eigen::VectorXd& nu);

        void simulate(Eigen::VectorXd& eta, Eigen::VectorXd& nu, Eigen::Vector2d& n, Eigen::Vector2d& n_c, double& psi_d, double& u_d, double& r_d, double& u_d_dot);

        public:
        EIGEN_MAKE_ALIGNED_OPERATOR_NEW

};

#endif