#ifndef EDGE_TURNING_RADIUS_H
#define EDGE_TURNING_RADIUS_H

#include "Vertex_Pose.hpp"
#include "Vertex_Time.hpp"
#include "Edge_TEB_Base.hpp"
#include "Penalty.hpp"
#include "../TEB_Config.hpp"

#include <cmath>

namespace teb_local_planner
{

class EdgeTurningRadius : public BaseTebMultiEdge<1, double>
{
public:

  /**
   * @brief Construct edge.
   */	      
  EdgeTurningRadius()
  {
    this->resize(3); // Since we derive from a g2o::BaseMultiEdge, set the desired number of vertices
  }
  
  /**
   * @brief Actual cost function
   */  
  void computeError()
  {
    const VertexPose* conf1 = static_cast<const VertexPose*>(_vertices[0]);
    const VertexPose* conf2 = static_cast<const VertexPose*>(_vertices[1]);
    const VertexTimeDiff* deltaT = static_cast<const VertexTimeDiff*>(_vertices[2]);
    
    const Eigen::Vector2d deltaS = conf2->estimate().position() - conf1->estimate().position();
    
    double dist = deltaS.norm();
    const double angle_diff = g2o::normalize_theta(conf2->theta() - conf1->theta());
    if (cfg_->trajectory.exact_arc_length && angle_diff != 0)
    {
        double radius =  dist/(2*sin(angle_diff/2));
        dist = fabs( angle_diff * radius ); // actual arg length!
    }
    double vel = dist / deltaT->estimate();
    
    const double omega = angle_diff / deltaT->estimate();

    if (fabs(omega) > 0)
    {
        double r = fabs(vel/omega);
        _error[0] = penaltyBoundFromBelow(r, cfg_->robot.min_turning_radius, cfg_->optim.penalty_epsilon);
    }
    else
    {
        _error[0] = 0;
    }   
  }
  public: 
  EIGEN_MAKE_ALIGNED_OPERATOR_NEW
};
}
#endif