#include "include/Simulation.hpp"

struct Config
{
    std::vector<PoseSE2> ASVs_eta0;
    std::vector<std::vector<Eigen::Vector2d>> ASVs_wps;
    std::vector<double> ASVs_nominal_speed;
};

int main()
{
    Ellipse e1;
    e1.position = Eigen::Vector2d(100,-100);
    e1.a = 80;
    e1.b = 80;
    e1.start = 90;
    e1.end = 180;

    Ellipse e2;
    e2.position = Eigen::Vector2d(-100, 0);
    e2.a = 30;
    e2.b = 40;
    e2.start = -90;
    e2.end = 90;

    Ellipse e3;
    e3.position = Eigen::Vector2d(-30, -50);
    e3.a = 30;
    e3.b = 15;
    e3.start = -180;
    e3.end = 180;
    e3.angle = 45;

    Ellipse e4;
    e4.position = Eigen::Vector2d(15, 40);
    e4.a = 50;
    e4.b = 25;
    e4.start = -180;
    e4.end = 180;

    auto map_details = std::vector<Ellipse>{e1,e2,e3,e4};
    std::vector<Config> cfgs;

    Config cfg1;
    cfg1.ASVs_eta0 = {PoseSE2(-90,-90,D2R(45))};
    cfg1.ASVs_nominal_speed = {2.5};
    cfg1.ASVs_wps.resize(1);
    cfg1.ASVs_wps[0] = {cfg1.ASVs_eta0[0].position(), Eigen::Vector2d(-40,-20), Eigen::Vector2d(40,0), Eigen::Vector2d(80,60)};
    cfgs.push_back(cfg1);

    Config cfg2;
    cfg2.ASVs_eta0 = {PoseSE2(-90,-90,D2R(45)), PoseSE2(-90,90,D2R(-45))};
    cfg2.ASVs_nominal_speed = {2.5, 2.5};
    cfg2.ASVs_wps.resize(2);
    cfg2.ASVs_wps[0] = {cfg2.ASVs_eta0[0].position(), Eigen::Vector2d(-40,-20), Eigen::Vector2d(40,0), Eigen::Vector2d(80,60)};
    cfg2.ASVs_wps[1] = {cfg2.ASVs_eta0[1].position(), Eigen::Vector2d(10,-30), Eigen::Vector2d(-40,-90)};
    cfgs.push_back(cfg2);

    Config cfg3;
    cfg3.ASVs_eta0 = {PoseSE2(-90,-90,D2R(45)), PoseSE2(-90,90,D2R(-45)), PoseSE2(65,-5,D2R(-180))};
    cfg3.ASVs_nominal_speed = {2.5, 2.5, 2.5};
    cfg3.ASVs_wps.resize(3);
    cfg3.ASVs_wps[0] = {cfg3.ASVs_eta0[0].position(), Eigen::Vector2d(-40,-20), Eigen::Vector2d(40,0), Eigen::Vector2d(80,60)};
    cfg3.ASVs_wps[1] = {cfg3.ASVs_eta0[1].position(), Eigen::Vector2d(10,-30), Eigen::Vector2d(-40,-90)};
    cfg3.ASVs_wps[2] = {cfg3.ASVs_eta0[2].position(), Eigen::Vector2d(-50,-10), Eigen::Vector2d(-90,70)};
    cfgs.push_back(cfg3);

    if (1)
    {
        auto cfg = cfg3;
        run_simulation(map_details, cfg.ASVs_eta0, cfg.ASVs_wps, cfg.ASVs_nominal_speed);
    }
    else
    {
        for (int i=1; i<cfgs.size(); ++i)
            run_simulation(map_details,cfgs[i].ASVs_eta0, cfgs[i].ASVs_wps, cfgs[i].ASVs_nominal_speed);
    }

    show();
}